# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import wx
import wx.xrc
import wx.lib.colourselect
import itertools

import Preferences

class ColourSelect(wx.lib.colourselect.ColourSelect):
    # wrapper to allow use in XRC, from http://www.nabble.com/ColourSelect-in-XRC-td17964472.html
    def __init__(self):
        self.PostCreate(wx.PreBitmapButton())
        self.SetLabel("")

    def Init(self, Callback=None):
        self.Bind(wx.EVT_BUTTON, self.OnClick)
        self.callback = Callback
        

class Controller(object):
    def __init__(self):
        import os.path
        self.res = wx.xrc.XmlResource(os.path.join('xrc', 'pygrid_prefs.xrc'))
        self.frame = self.res.LoadDialog(None, 'mainFrame')
        self.frame.Bind(wx.EVT_CLOSE, lambda event: self.hideWindow())
        
        self.player_colorselectors = {1: wx.xrc.XRCCTRL(self.frame, 'p1colorselect'), 2: wx.xrc.XRCCTRL(self.frame, 'p2colorselect')}
        self.board_colorselectors = {0: wx.xrc.XRCCTRL(self.frame, 'b1colorselect'), 1: wx.xrc.XRCCTRL(self.frame, 'b2colorselect')}
        self.board_alphaslider = wx.xrc.XRCCTRL(self.frame, 'b1transparency')
        
        # initialize color pickers and sliders and set initial values
        # note lambdas use default argument to get *current* value of loop variable
        for pnum, csel in self.player_colorselectors.iteritems():
            csel.Init()
            csel.Bind(wx.lib.colourselect.EVT_COLOURSELECT, lambda event,i=pnum: self.player_color_changed(i))
        
        for bnum, csel in self.board_colorselectors.iteritems():
            csel.Init()
            csel.Bind(wx.lib.colourselect.EVT_COLOURSELECT, lambda event: self.board_color_changed())
            
        self.board_alphaslider.SetMin(0)
        self.board_alphaslider.SetMax(255)
        self.board_alphaslider.Bind(wx.EVT_SCROLL, lambda event: self.board_color_changed(slider=True))
                        
        self.animation_checkbox = wx.xrc.XRCCTRL(self.frame, 'animation_checkbox')
        self.animation_checkbox.Bind(wx.EVT_CHECKBOX, self.toggle_animation)

        self.checkupdate_checkbox = wx.xrc.XRCCTRL(self.frame, 'checkupdate_checkbox')
        self.checkupdate_checkbox.Bind(wx.EVT_CHECKBOX, self.toggle_checkforupdate)
        
        self.username_field = wx.xrc.XRCCTRL(self.frame, 'name_field')
        self.username_field.Bind(wx.EVT_TEXT, self.update_username) # this updates on every keypress
        
        self.set_controls_from_prefs()

        wx.xrc.XRCCTRL(self.frame, 'restore_defaults_button').Bind(wx.EVT_BUTTON, self.restore_defaults)        
        wx.xrc.XRCCTRL(self.frame, 'mainPanel').GetSizer().Fit(self.frame)
        
    def set_controls_from_prefs(self):
        for pnum, csel in self.player_colorselectors.iteritems():
            csel.SetColour(wx.Colour(*Preferences.prefs.player_color(pnum)))
        for bnum, csel in self.board_colorselectors.iteritems():
            csel.SetColour(wx.Colour(*(Preferences.prefs.board_color(bnum)[:3])))
            
        self.board_alphaslider.SetValue(Preferences.prefs.board_color(0)[3])
        
        self.animation_checkbox.SetValue(Preferences.prefs.use_animation())
        self.checkupdate_checkbox.SetValue(Preferences.prefs.check_for_update())
        self.username_field.SetValue(Preferences.prefs.get_username())
        
    def showWindow(self):
        self.frame.Show()
        self.frame.Raise()

    def hideWindow(self):
        self.frame.Hide()

    def player_color_changed(self, pnum):
        Preferences.prefs.set_player_color(self.player_colorselectors[pnum].GetColour().Get(), pnum)
        self.wx_color_select_fix()
        
    def board_color_changed(self, slider=False):
        # combine color from ColourSelect control and transparency from slider
        for bnum in self.board_colorselectors:
            color = list(self.board_colorselectors[bnum].GetColour().Get()) + [self.board_alphaslider.GetValue()]
            Preferences.prefs.set_board_color(color, bnum)
        if not slider:
            self.wx_color_select_fix()
        
    def toggle_animation(self, event):
        Preferences.prefs.set_animation(self.animation_checkbox.GetValue())
        
    def toggle_checkforupdate(self, event):
        Preferences.prefs.set_check_for_update(self.checkupdate_checkbox.GetValue())
        
    def update_username(self, event):
        Preferences.prefs.set_username(self.username_field.GetValue())
        
    def restore_defaults(self, event):
        Preferences.prefs.clear_board_settings()
        self.set_controls_from_prefs()
        
    # Earlier versions of wxPython (at least the 2.8.4.0 that ships with Leopard) have a bug which causes a crash if
    # a ColourDialog is shown twice for the same window. Oddly, hiding and showing the window is a workaround.
    # This method is called after selecting a color; it checks the wxPython version and does the hide/show if needed.
    def wx_color_select_fix(self):
        try:
            # wx.__version__ will be a string like '2.8.9.1'
            vers = [int(part) for part in wx.__version__.split('.')]
            dofix = (vers <= [2,8,4,0])
        except:
            dofix = True
        if dofix:
            self.hideWindow()
            self.showWindow()
        